from abc import ABC

from chaosmesh.experiments.base.k8s.podfault import PodChaos


class BasePodKillExperiment(PodChaos, ABC):
    """
    The BasePodKillExperiment class is a subclass of the PodChaos class used to implement a chaos experiment that simulates
    the killing of a pod.

    Parameters
    ----------
    **kwargs : any number of keyword arguments

    """

    def __init__(self, **kwargs):
        """
        Initialize the BasePodKillExperiment class by passing any keyword arguments to the super class's __init__ method.

        """
        super(BasePodKillExperiment, self).__init__(**kwargs)

    def action(self) -> str:
        """
        Returns the string "pod-kill" to indicate that this type of chaos experiment involves simulating the killing of a pod.

        Returns
        -------
        str
            "pod-kill"

        """
        return "pod-kill"



from abc import ABC, abstractmethod

from kubernetes import client

from chaosmesh.k8s.kubeclient import load_kube_config


class K8SResource(ABC):
    """
    Base class for all the k8s resources
    """

    def __init__(self):
        load_kube_config()

    @abstractmethod
    def client(self):
        """
        Returns the k8s client
        :return: 
        """
        pass

    @property
    def k8sclient(self):
        """
        Returns the kubernetes cluster client
        :return: 
        """
        return client



from abc import ABC

from chaosmesh.experiments.base.hosts.disk import DiskFault


class BaseReadPayloadExperiment(DiskFault, ABC):
    """
    This class represents a base read payload experiment using the DiskFault and ABC classes.
    """

    def __init__(self, **kwargs):
        """
        Initializes a new instance of the BaseReadPayloadExperiment class.
        :param kwargs: Optional keyword arguments for configuring the experiment.
        """
        super(BaseReadPayloadExperiment, self).__init__(**kwargs)

    def action(self) -> str:
        """
        Returns the name of the action performed by the experiment.
        :return: A string representing the name of the action.
        """
        return "disk-read-payload"

    def spec(self, namespace, name) -> dict:
        """
        Returns a dictionary containing the experiment specification.
        :param namespace: The Kubernetes namespace in which to create the experiment.
        :param name: The name of the experiment.
        :return: A dictionary containing the experiment specification.
        """
        return {
            "action": self.kwargs['action'],
            "address": self.kwargs['address'],
            "disk-read-payload": {
                "size": self.kwargs['size'],
                "path": self.kwargs['path'],
                "payload-process-num": self.kwargs['payload_process_num'],
            },
            "duration": self.kwargs['duration'],
        }



from abc import ABC

from chaosmesh.experiments.base.hosts.stress import StressTest


class BaseHostsStressMemoryExperiment(StressTest, ABC):
    """
    Base class for hosting stress memory experiments

    This class is a subclass of StressTest and implements the `action` and `spec` methods
    for stress memory experiments. The class also defines a `validate` method to validate
    the inputs provided to the class.
    """

    def __init__(self, **kwargs):
        """
        Initialize the BaseHostsStressMemoryExperiment class

        :param kwargs: Keyword arguments to pass to the parent class StressTest
        """
        super(BaseHostsStressMemoryExperiment, self).__init__(**kwargs)

    def action(self) -> str:
        """
        Return the action name for the stress memory experiment

        :return: String "stress-mem"
        """
        return "stress-mem"

    def validate(self):
        """
        Validate the inputs for the stress memory experiment

        This method checks that the required parameters, `address` and `size`, are provided
        and are of the correct type.
        """
        assert self.kwargs['address'] is not None, "address cannot be None"
        assert self.kwargs['size'] is not None, "size cannot be None"

    def spec(self, namespace, name):
        """
        Return the specification for the stress memory experiment

        :param namespace: Namespace for the experiment
        :param name: Name of the experiment

        :return: Dictionary containing the specification for the stress memory experiment
        """
        return {
            "action": self.action(),
            "address": self.kwargs['address'],
            "mode": self.kwargs.get('mode'),
            "stress-mem": {
                "size": self.kwargs['size']
            },
            "duration": self.kwargs['duration']
        }



import logging

from chaosmesh.experiments import Experiment
from chaosmesh.experiments.factory import ExperimentFactory
from chaosmesh.schedules import Schedule

log = logging.getLogger("chaosmesh")


class Client:
    """
    The `Client` class provides the interface to interact with the chaos mesh experiments.

    Attributes:
        factory (ExperimentFactory): An instance of `ExperimentFactory` class to get the experiment instance.

    Methods:
        start_experiment (Experiment, str, str, **kwargs) -> None:
            Starts the specified chaos experiment.

        pause_experiment (Experiment, str, str, **kwargs) -> None:
            Pauses the specified chaos experiment.

        delete_experiment (Experiment, str, str, **kwargs) -> None:
            Deletes the specified chaos experiment.
    """

    def __init__(self, version):
        """
        The constructor for the `Client` class.

        Arguments:
            version (str): The version of chaos mesh to be used.
        """
        self.factory = ExperimentFactory.get_instance(version)

    def start_experiment(self, experiment_type: Experiment, namespace: str, name: str, **kwargs):
        """
        Starts the specified chaos experiment.

        Arguments:
            experiment_type (Experiment): The type of chaos experiment to start.
            namespace (str): The namespace in which the experiment is to be run.
            name (str): The name of the chaos experiment.
            **kwargs: The optional parameters for the experiment.

        Returns:
            dict: The applied experiment's resource in Kubernetes.
        """
        log.info(f"starting {experiment_type.value} experiment {name} in {namespace} namespace")
        return self.factory.get_experiment(experiment_type, **kwargs).submit(namespace, name)

    def pause_experiment(self, experiment_type: Experiment, namespace: str, name: str, **kwargs) -> None:
        """
        Pauses the specified chaos experiment.

        Arguments:
            experiment_type (Experiment): The type of chaos experiment to pause.
            namespace (str): The namespace in which the experiment is running.
            name (str): The name of the chaos experiment.
            **kwargs: The optional parameters for the experiment.

        Returns:
            None
        """
        log.info(f"pausing {experiment_type.value} experiment {name} in {namespace} namespace")
        self.factory.get_experiment(experiment_type, **kwargs).pause(namespace, name)

    def delete_experiment(self, experiment_type: Experiment, namespace: str, name: str, **kwargs):
        """
        Deletes the specified chaos experiment.

        Arguments:
            experiment_type (Experiment): The type of chaos experiment to delete.
            namespace (str): The namespace in which the experiment is running.
            name (str): The name of the chaos experiment.
            **kwargs: The optional parameters for the experiment.

        Returns:
            dict: The deleted experiment's resource in Kubernetes.
        """
        log.info(f"deleting {experiment_type.value} experiment {name} in {namespace} namespace")
        return self.factory.get_experiment(experiment_type, **kwargs).delete(namespace=namespace, name=name)

    def schedule_experiment(self, experiment_type: Experiment, namespace: str, name: str, cron_schedule: str, **kwargs):
        log.info(f"scheduling {experiment_type.value} experiment {name} in {namespace} namespace")
        return Schedule(experiment=self.factory.get_experiment(experiment_type, **kwargs), schedule=cron_schedule, **kwargs).submit(namespace=namespace, name=name)

    def delete_schedule(self, experiment_type: Experiment, namespace: str, name: str, **kwargs):
        return Schedule(self.factory.get_experiment(experiment_type, **kwargs)).delete(namespace=namespace, name=name)



from chaosmesh.experiments.base.hosts.stress.memory import BaseHostsStressMemoryExperiment


class HostsStressMemory(BaseHostsStressMemoryExperiment):
    """
    This class creates a memory stress experiment for k8s hosts in ChaosMesh.

    It is a subclass of BaseHostsStressMemoryExperiment and inherits all the attributes and methods from it.

    Args:
        **kwargs: Keyword arguments that can be used to initialize the attributes of the parent class.
    """

    def __init__(self, **kwargs):
        """
        The constructor method that initializes the attributes of the class and its parent class.

        Args:
            **kwargs: Keyword arguments that can be used to initialize the attributes of the parent class.
        """
        super(HostsStressMemory, self).__init__(**kwargs)



from chaosmesh.experiments.base.hosts.disk.read_payload import BaseReadPayloadExperiment


class ReadPayload(BaseReadPayloadExperiment):
    """
    A subclass of BaseReadPayloadExperiment class that represents a disk read payload experiment.

    This class inherits properties and methods from the BaseReadPayloadExperiment class and overrides its constructor.
    """

    def __init__(self, **kwargs):
        """
        Initializes the ReadPayload object with the arguments passed to it.

        Args:
            **kwargs: Arbitrary keyword arguments. These arguments are used to set the attributes of the ReadPayload
            object.
        """
        super(ReadPayload, self).__init__(**kwargs)



from abc import ABC

from chaosmesh.experiments.base.k8s.jvmfault import JVMFault
from chaosmesh.k8s.selector import Selector


class BaseRaiseExceptionExperiment(JVMFault, ABC):
    """
    This class represents the base experiment to raise exceptions in a JVM application.

    It inherits from the `JVMFault` class and provides the default parameters for the experiment.
    The experiment raises exceptions in the target class and method of the JVM application.

    Attributes:
        kwargs (dict): Keyword arguments to initialize the `JVMFault` class.
    """

    def __init__(self, **kwargs):
        """
        Initialize the `BaseRaiseExceptionExperiment` class.

        Calls the `__init__` method of the `JVMFault` class with the provided keyword arguments.
        """
        super(BaseRaiseExceptionExperiment, self).__init__(**kwargs)

    @property
    def defaults(self):
        """
        Returns a dictionary with the default parameters for the experiment.

        The default parameters include:
        - `action`: The type of the experiment, which is set to "exception".
        - `cpuCount`: The number of CPU cores to be affected by the experiment.
        - `duration`: The duration of the experiment.
        - `latency`: The latency to introduce in the application.
        - `memType`: The type of memory to be affected by the experiment.
        - `mode`: The mode of the experiment, which is set to "all".
        - `ruleData`: The rule data for the experiment.
        - `value`: The value for the experiment.

        Returns:
            dict: The default parameters for the experiment.
        """
        return {
            "action": self.action(),
            "cpuCount": 0,
            "duration": "",
            "latency": 0,
            "memType": "",
            "mode": "all",
            "ruleData": "",
            "value": ""
        }

    def action(self) -> str:
        """
        Returns the type of the experiment, which is set to "exception".

        Returns:
            str: The type of the experiment, which is set to "exception".
        """
        return "exception"

    def validate(self):
        """
        Validates the parameters of the experiment.

        Raises:
            AssertionError: If the `targetClass` parameter is None.
            AssertionError: If the `method` parameter is None.
            AssertionError: If the `exception` parameter is None.
            AssertionError: If the `port` parameter is None.
            AssertionError: If the `selector` parameter is None.
            AssertionError: If the `selector` parameter is not of type `Selector`.

        """
        assert self.kwargs['targetClass'] is not None, "`targetClass` cannot be None"
        assert self.kwargs['method'] is not None, "method cannot be None"
        assert self.kwargs['exception'] is not None, "exception cannot be None"
        assert self.kwargs['port'] is not None, "port cannot be None"

        assert self.kwargs['selector'] is not None, "label selector cannot be None"
        assert isinstance(self.kwargs['selector'], Selector), "invalid Selector type, should be of type k8s.selector"



from chaosmesh.experiments.base.k8s.podfault.pod_failure import BasePodFailureExperiment


class PodFailure(BasePodFailureExperiment):
    """
    A class for creating a PodFailure chaos experiment in ChaosMesh.

    This class is a subclass of `BasePodFailureExperiment` and provides an implementation for creating a chaos experiment that causes a Pod to fail.

    Attributes:
        kwargs (dict): A dictionary of arguments for the chaos experiment.
    """

    def __init__(self, **kwargs):
        """
        Initializes the `PodFailure` class with the given arguments.

        Args:
            kwargs (dict): A dictionary of arguments for the chaos experiment.

        """
        super(PodFailure, self).__init__(**kwargs)



from abc import ABC

from chaosmesh.experiments.base.hosts.stress import StressTest


class BaseHostsStressCPUExperiment(StressTest, ABC):
    """
    BaseHostsStressCPUExperiment class is a derived class from StressTest and is used to perform CPU stress tests on a host.

    Attributes:
    -----------
    kwargs : dict
        A dictionary of arguments to initialize the experiment.

    Methods:
    --------
    action() -> str:
        Returns the action string "stress-cpu".

    validate() -> None:
        Validates the inputs in the kwargs dictionary to make sure all required parameters are present.
        Raises an assertion error if the address, load, or workers are None.

    spec(namespace: str, name: str) -> dict:
        Returns a dictionary representation of the experiment.
        The dictionary contains information about the action, address, mode, stress-cpu and the duration.
    """

    def __init__(self, **kwargs):
        """
        Constructor to initialize the BaseHostsStressCPUExperiment class.

        Parameters:
        -----------
        kwargs : dict
            A dictionary of arguments to initialize the experiment.
        """
        super(BaseHostsStressCPUExperiment, self).__init__(**kwargs)

    def action(self) -> str:
        """
        Returns the action string "stress-cpu".

        Returns:
        --------
        str:
            The string "stress-cpu".
        """
        return "stress-cpu"

    def validate(self) -> None:
        """
        Validates the inputs in the kwargs dictionary to make sure all required parameters are present.
        Raises an assertion error if the address, load, or workers are None.
        """
        assert self.kwargs['address'] is not None, "address cannot be None"

        assert self.kwargs['load'] is not None, "load cannot be None"
        assert isinstance(self.kwargs['load'], int), "load type should be int"

        assert self.kwargs['workers'] is not None, "workers cannot be None"

    def spec(self, namespace, name) -> dict:
        """
        Returns a dictionary representation of the experiment.
        The dictionary contains information about the action, address, mode, stress-cpu and the duration.

        Parameters:
        -----------
        namespace: str
            Namespace to run the experiment in.

        name: str
            Name of the experiment.

        Returns:
        --------
        dict:
            Dictionary representation of the experiment.
        """
        return {
            "action": self.action(),
            "address": self.kwargs['address'],
            "mode": self.kwargs.get('mode'),
            "stress-cpu": {
                "load": self.kwargs['load'],
                "workers": self.kwargs['workers']
            },
            "duration": self.kwargs['duration']
        }



from abc import ABC
from dataclasses import asdict

from chaosmesh.k8s.experiment import ChaosExperiment


class JVMFault(ChaosExperiment, ABC):
    """
    JVMFault is a subclass of the `ChaosExperiment` class and implements the abstract base class `ABC`.

    The class provides the functionality for conducting JVM-level chaos experiments.

    Attributes:
        kwargs (dict): A dictionary of keyword arguments passed to the parent class.

    Properties:
        defaults: A generator that yields default values for the JVMFault experiment.

    Methods:
        action: Returns the action that will be performed in the JVMFault experiment.
        api_resources: Returns the API resource specifications for the JVMFault experiment.
        spec: Returns the specifications for the JVMFault experiment.
    """

    def __init__(self, **kwargs):
        """
        Initializes the JVMFault object.

        Args:
            kwargs (dict): A dictionary of keyword arguments passed to the parent class.
        """
        super(JVMFault, self).__init__(**kwargs)

    @property
    def schedule(self) -> dict:
        return {
            "type": "JVMChaos",
            "spec": "jvmChaos"
        }

    @property
    def defaults(self) -> dict:
        """
        A generator that yields default values for the JVMFault experiment.

        Returns:
            A generator that yields default values for the JVMFault experiment.
        """
        yield

    def action(self) -> str:
        """
        Returns the action that will be performed in the JVMFault experiment.

        Returns:
            str: A string representing the action to be performed.
        """
        pass

    def api_resources(self) -> dict:
        """
        Returns the API resource specifications for the JVMFault experiment.

        Returns:
            dict: A dictionary containing the API resource specifications for the JVMFault experiment.
        """
        return {"group": 'chaos-mesh.org', "version": 'v1alpha1', "plural": "jvmchaos"}

    def spec(self, namespace, name) -> dict:
        """
        Returns the specifications for the JVMFault experiment.

        Args:
            namespace (str): The namespace in which the JVMFault experiment will be executed.
            name (str): The name of the JVMFault experiment.

        Returns:
            dict: A dictionary containing the specifications for the JVMFault experiment.
        """
        return {
            "selector": asdict(self.kwargs['selector']),
            "mode": self.kwargs.get('mode'),
            "duration": self.kwargs.get('duration'),
            "action": self.action(),
            "name": name,
            "class": self.kwargs.get('targetClass'),
            "method": self.kwargs.get('method'),
            "value": self.kwargs.get('value'),
            "exception": self.kwargs.get('exception'),
            "latency": self.kwargs.get('latency'),
            "cpuCount": self.kwargs.get('cpuCount'),
            "memType": self.kwargs.get('memType'),
            "port": self.kwargs['port'],
            "ruleData": self.kwargs['ruleData'],
        }



from chaosmesh.experiments.base.k8s.network.bandwidth import BaseNetworkBandwidthExperiment


class NetworkBandwidth(BaseNetworkBandwidthExperiment):
    """
    NetworkBandwidth is a subclass of NetworkBandwidthExperiment, which defines a Kubernetes network chaos
    experiment that modifies network bandwidth.
    """

    def __init__(self, **kwargs):
        """
        Initializes a new instance of the NetworkBandwidth class with the specified keyword arguments.

        Args:
            **kwargs: A dictionary of keyword arguments that are passed to the NetworkBandwidthExperiment constructor.

        Returns:
            A new instance of the NetworkBandwidth class.
        """
        super(NetworkBandwidth, self).__init__(**kwargs)



from abc import ABC

from chaosmesh.k8s.experiment import ChaosExperiment


class StressTest(ChaosExperiment, ABC):

    def __init__(self, **kwargs):
        super(StressTest, self).__init__(**kwargs)

    @property
    def defaults(self) -> dict:
        return {
            "duration": '',
            "labels": {},
            "load": 100,
            "mode": "all",
            "size": '10',
            "workers": 1
        }

    @property
    def schedule(self) -> dict:
        return {
            "type": "StressChaos",
            "spec": "stressChaos"
        }

    def api_resources(self) -> dict:
        return {"group": 'chaos-mesh.org', "version": 'v1alpha1', "plural": "stresschaos"}



from chaosmesh.experiments.base.k8s.network.partition import BaseNetworkPartitionExperiment


class NetworkPartition(BaseNetworkPartitionExperiment):
    """
    A network partition experiment for Kubernetes.
    """

    def __init__(self, **kwargs):
        """
        Initialize the NetworkPartition.

        :param kwargs: keyword arguments for the experiment.
        """

        super(NetworkPartition, self).__init__(**kwargs)



from abc import ABC
from dataclasses import asdict

from chaosmesh.experiments.base.k8s.podfault import PodChaos
from chaosmesh.k8s.selector import Selector


class BaseContainerKillExperiment(PodChaos, ABC):
    """
    Subclass of PodChaos that implements a chaos experiment that kills one or more containers in a
    pod by their name.
    """

    def __init__(self, **kwargs):
        """
        Initializes the BaseContainerKillExperiment class by calling the super class's __init__ method.
        :param kwargs: Keyword arguments that are passed to the super class.
        """
        super(BaseContainerKillExperiment, self).__init__(**kwargs)

    def action(self) -> str:
        """
        Returns the type of chaos action to be performed, which is 'container-kill' in this case.
        :return: The type of chaos action as a string.
        """
        return "container-kill"

    def validate(self) -> None:
        """
        Validates the arguments passed to the experiment by checking that the label selector is not None and
        of type Selector. Additionally, the container name must not be None.
        """
        assert self.kwargs['selector'] is not None, "label selector cannot be None"
        assert isinstance(self.kwargs['selector'], Selector), "check the selector type"

        assert self.kwargs['container_names'] is not None, "container name cannot be None"

    def spec(self, namespace, name) -> dict:
        """
        Returns the specifications for the chaos experiment, including the selector, mode, container names,
        action and grace period.
        :param namespace: The namespace in which the experiment is to be performed.
        :param name: The name of the experiment.
        :return: A dictionary containing the specifications for the chaos experiment.
        """
        return {
            "selector": asdict(self.kwargs['selector']),
            "mode": self.kwargs['mode'],
            "containerNames": self.kwargs['container_names'],
            "action": self.kwargs.get('action'),
            "gracePeriod": self.kwargs.get('gracePeriod')
        }



"""
This code defines a `Schedule` class that represents a Chaos Mesh schedule.

A schedule is a recurring event that can be used to run Chaos Mesh experiments.

The `Schedule` class has the following attributes:

* `experiment`: The Chaos Mesh experiment that this schedule represents.
* `kwargs`: Any additional keyword arguments that are passed to the constructor.

The `Schedule` class has the following methods:

* `api_resources()`: Returns the API resources for this schedule.
* `validate()`: Validates the schedule.
* `defaults()`: Returns the default values for this schedule.
* `spec()`: Returns the specification for this schedule.
* `create()`: Creates this schedule.

"""

import json
import logging

from chaosmesh.k8s.chaos_mesh import ChaosMesh
from chaosmesh.k8s.experiment import ChaosExperiment

log = logging.getLogger("chaosmesh")


class Schedule(ChaosMesh):

    """
    Represents a Chaos Mesh schedule.

    Args:
        experiment: The Chaos Mesh experiment that this schedule represents.
        **kwargs: Any additional keyword arguments that are passed to the constructor.
    """

    def __init__(self, experiment: ChaosExperiment, **kwargs):
        """
        Initializes a new `Schedule` object.

        Args:
            experiment: The Chaos Mesh experiment that this schedule represents.
            **kwargs: Any additional keyword arguments that are passed to the constructor.
        """
        self.experiment = experiment
        super(Schedule, self).__init__(**kwargs)

    def api_resources(self) -> dict:
        """
        Returns the API resources for this schedule.

        Returns:
            A dictionary that contains the API group, version, and plural for this schedule.
        """
        return {"group": 'chaos-mesh.org', "version": 'v1alpha1', "plural": "schedules"}

    def validate(self) -> None:
        """
        Validates the schedule.

        This method ensures that the schedule is valid and can be created.
        """
        assert self.kwargs['schedule'] is not None, "schedule cannot be None"

    @property
    def defaults(self) -> dict:
        """
        Returns the default values for this schedule.

        Returns:
            A dictionary that contains the default values for this schedule.
        """
        return {
            "historyLimit": 3,
            "concurrencyPolicy": 'Forbid',
            "startingDeadlineSeconds": None,
        }

    def spec(self, namespace, name) -> dict:
        """
        Returns the specification for this schedule.

        The specification includes the schedule name, the experiment name, and the experiment spec.

        Args:
            namespace: The namespace where the schedule will be created.
            name: The name of the schedule.

        Returns:
            A dictionary that contains the specification for this schedule.
        """
        return {
            "schedule": self.kwargs.get('schedule'),
            "startingDeadlineSeconds": self.kwargs.get('startingDeadlineSeconds'),
            "concurrencyPolicy": self.kwargs.get('concurrencyPolicy'),
            "historyLimit": self.kwargs.get('historyLimit'),
            "type": self.experiment.schedule['type'],
            self.experiment.schedule['spec']: self.experiment.spec(namespace=namespace, name=name),
        }

    def create(self, name, namespace, labels=None):
        """
        Creates this schedule.

        Args:
            name: The name of the schedule.
            namespace: The namespace where the schedule will be created.
            labels: Any labels that should be applied to the schedule.

        Returns:
            The created schedule object.
        """
        manifest = self.manifest(namespace=namespace, name=name, labels=labels)

        log.debug(f"creating chaosmesh resource {json.dumps(manifest, indent=4)}")

        applied = super().apply(namespace=namespace, object=manifest)

        log.info(f"chaosmesh experiment {name} got applied")

        return applied



from chaosmesh.experiments.base.hosts.disk.fill import BaseFillExperiment


class Fill(BaseFillExperiment):
    """
    A class for running disk fill experiments with the specified parameters.
    """

    def __init__(self, **kwargs):
        """
        Initializes the experiment with the provided keyword arguments.

        Args:
            **kwargs: Keyword arguments specifying the parameters of the experiment.
        """
        super(Fill, self).__init__(**kwargs)



from enum import Enum


class Experiment(Enum):
    """An enumeration class that represents the different types of experiments that can be run.

    Attributes:
        POD_FAILURE (str): Indicates a Pod failure experiment.
        POD_KILL (str): Indicates a Pod kill experiment.
        CONTAINER_KILL (str): Indicates a Container kill experiment.
        POD_STRESS_CPU (str): Indicates a Pod stress CPU experiment.
        POD_STRESS_MEMORY (str): Indicates a Pod stress memory experiment.
        RAISE_EXCEPTION (str): Indicates an experiment that raises an exception.
        GC (str): Indicates a garbage collection experiment.
        HOST_STRESS_MEMORY (str): Indicates a Host stress memory experiment.
        HOST_STRESS_CPU (str): Indicates a Host stress CPU experiment.
        NETWORK_PARTITION (str): Indicates Network Partition experiment.
        NETWORK_BANDWIDTH (str): Indicates Network Bandwidth experiment.
        HOST_READ_PAYLOAD (str): Indicates Host disk fault read payload experiment.
        HOST_WRITE_PAYLOAD (str): Indicates Host disk fault write payload experiment.
        HOST_FILL (str): Indicates Host disk fill experiment.
    """
    POD_FAILURE = "POD_FAILURE"
    POD_KILL = "POD_KILL"
    CONTAINER_KILL = "CONTAINER_KILL"

    POD_STRESS_CPU = "POD_STRESS_CPU"
    POD_STRESS_MEMORY = "POD_STRESS_MEMORY"

    RAISE_EXCEPTION = "RAISE_EXCEPTION"
    GC = "GC"

    HOST_STRESS_MEMORY = "HOST_STRESS_MEMORY"
    HOST_STRESS_CPU = "HOST_STRESS_CPU"

    NETWORK_PARTITION = "NETWORK_PARTITION"
    NETWORK_BANDWIDTH = "NETWORK_BANDWIDTH"

    HOST_READ_PAYLOAD = "HOST_READ_PAYLOAD"
    HOST_WRITE_PAYLOAD = "HOST_WRITE_PAYLOAD"
    HOST_DISK_FILL = "HOST_DISK_FILL"



from abc import ABC

from chaosmesh.k8s.crd import CustomObjectsApi


class ChaosMesh(CustomObjectsApi, ABC):
    """
    A base class for Chaos Mesh experiments. Inherits from CustomObjectsApi, which provides functionality for working with
    Kubernetes custom resources.

    """

    def __init__(self, **kwargs):
        """
        Initializes a new instance of the ChaosMesh class.

        Args:
            **kwargs: Additional keyword arguments.

        """
        self.kwargs = kwargs
        super(ChaosMesh, self).__init__()

        # initialize defaults
        for key, value in self.defaults.items():
            self.kwargs[key] = kwargs.get(key, value)

    @property
    def defaults(self) -> dict:
        """
        The default values for the experiment.

        Returns:
            dict: The default values.

        """
        yield

    def validate(self) -> None:
        """
        Validate the experiment's specification before submitting it to Kubernetes.

        """
        pass

    def submit(self, namespace, name, labels=None):
        """
        Submit an experiment to Kubernetes.

        Args:
            namespace (str): The namespace in which to create the experiment.
            name (str): The name of the experiment.
            labels (dict): The labels to apply to the experiment.

        Returns:
            dict: The created experiment.

        Raises:
            AssertionError: If namespace or name is None.

        """
        assert namespace is not None, "namespace can not be None"
        assert name is not None, "name can not be None"

        # validating the spec before applying it to k8s
        self.validate()

        return self.create(name=name, namespace=namespace, labels=labels)

    def create(self, name, namespace, labels=None):
        """
        Create an experiment in Kubernetes.

        Args:
            name (str): The name of the experiment.
            namespace (str): The namespace in which to create the experiment.
            labels (dict): The labels to apply to the experiment.

        """
        pass



from abc import ABC

from chaosmesh.k8s.experiment import ChaosExperiment


class StressTest(ChaosExperiment, ABC):
    """
    The StressTest is a class for defining a stress testing chaos experiment on a physical machine.

    Attributes:
    -----------
    None

    Methods:
    --------
    api_resources() -> dict:
        Returns a dictionary containing the Kubernetes API resource information for the experiment.
        
    defaults() -> dict:
        Returns a dictionary containing the default values for the experiment's parameters.
        
    action() -> str:
        Returns a string representing the type of action that the experiment performs.
    """

    def __init__(self, **kwargs):
        """
        Initializes a new instance of the StressTest class.

        Parameters:
        -----------
        kwargs : dict
            A dictionary containing the parameters for the experiment.
        """
        super(StressTest, self).__init__(**kwargs)

    @property
    def schedule(self) -> dict:
        return {
            "type": "PhysicalMachineChaos",
            "spec": "physicalMachineChaos"
        }

    def api_resources(self) -> dict:
        """
        Returns a dictionary containing the Kubernetes API resource information for the experiment.

        Returns:
        --------
        dict
            A dictionary containing the Kubernetes API resource information for the experiment.
        """
        return {"group": 'chaos-mesh.org', "version": 'v1alpha1', "plural": "physicalmachinechaos"}

    @property
    def defaults(self) -> dict:
        """
        Returns a dictionary containing the default values for the experiment's parameters.

        Returns:
        --------
        dict
            A dictionary containing the default values for the experiment's parameters.
        """
        return {
            "duration": "",
            "mode": "all",
            "workers": 1
        }

    def action(self) -> str:
        """
        Returns a string representing the type of action that the experiment performs.

        Returns:
        --------
        str
            A string representing the type of action that the experiment performs.
        """
        pass



from abc import ABC
from dataclasses import asdict

from chaosmesh.experiments.base.k8s.network import NetworkChaos
from chaosmesh.k8s.selector import Selector


class BaseNetworkBandwidthExperiment(NetworkChaos, ABC):
    """
    Defines a network bandwidth chaos experiment.

    Extends:
        NetworkChaos: An abstract class that provides common methods for network-related chaos experiments.
        ABC: An abstract base class that provides a common API for classes that are designed to be subclassed.

    Args:
        **kwargs: Arbitrary keyword arguments to configure the network bandwidth experiment.

    Attributes:
        defaults (dict): A dictionary of default values for the experiment configuration.

    """

    def __init__(self, **kwargs):
        """
        Initializes a new instance of the NetworkBandwidthExperiment class.

        Args:
            **kwargs: Arbitrary keyword arguments to configure the network bandwidth experiment.
        """
        super(BaseNetworkBandwidthExperiment, self).__init__(**kwargs)

    @property
    def defaults(self):
        """
        Gets the default values for the experiment configuration.

        Returns:
            dict: A dictionary of default values for the experiment configuration.
        """
        return {
            "action": self.action(),
            "duration": "",
            "minburst": 0,
            "mode": "all",
            "peakrate": 0
        }

    def validate(self) -> None:
        """
        Validates the configuration of the experiment.

        Raises:
            AssertionError: If the configuration is invalid.
        """
        assert self.kwargs['selector'] is not None, "label selector cannot be None"
        assert isinstance(self.kwargs['selector'], Selector), "invalid Selector type, should be of type k8s.selector"

        assert self.kwargs['rate'] is not None, "rate cannot be None, the rate allows are bps, kbps, mbps, gbps, tbps unit. For example, bps means bytes per second"

        assert self.kwargs['buffer'] is not None, "buffer cannot be None"
        assert self.kwargs.get('buffer', 0) > 0, "buffer should be greater than or equal to 1"

        assert self.kwargs['limit'] is not None, "limit cannot be None"
        assert self.kwargs.get('limit', 0) > 0, "limit should be greater than or equal to 1"

        assert self.kwargs['direction'] is not None, "direction cannot be None, allowed values are from, to, both"

        assert self.kwargs['external_targets'] is not None, "external_targets cannot be None"
        assert isinstance(self.kwargs['external_targets'], list), "external_targets should be of type list"

    def action(self) -> str:
        """
        Gets the action string for the experiment.

        Returns:
            str: The action string for the experiment.
        """
        return "bandwidth"

    def spec(self, namespace, name) -> dict:
        """
        Gets the experiment specification in dictionary format.

        Args:
            namespace (str): The namespace of the experiment.
            name (str): The name of the experiment.

        Returns:
            dict: The experiment specification in dictionary format.
        """
        return {
            "selector": asdict(self.kwargs['selector']),
            "mode": self.kwargs.get('mode'),
            "action": self.kwargs.get('action'),
            "duration": self.kwargs.get('duration'),
            "bandwidth": {
                "rate": self.kwargs.get('rate'),
                "buffer": self.kwargs.get('buffer'),
                "limit": self.kwargs.get('limit'),
                "peakrate": self.kwargs.get('peakrate'),
                "minburst": self.kwargs.get('minburst'),
            },
            "direction": self.kwargs.get('direction'),
            "externalTargets": self.kwargs['external_targets']
        }



from abc import ABC

from chaosmesh.experiments.base.hosts.disk import DiskFault


class BaseWritePayloadExperiment(DiskFault, ABC):
    """
    A class for implementing a base experiment that writes a payload to a disk.

    Args:
        DiskFault (class): A class that simulates disk faults.
        ABC (class): A class that represents an abstract base class.

    """

    def __init__(self, **kwargs):
        """
        Initializes a new instance of the BaseWritePayloadExperiment class.

        Args:
            **kwargs (dict): A dictionary of keyword arguments.

        """
        super(BaseWritePayloadExperiment, self).__init__(**kwargs)

    def action(self) -> str:
        """
        Returns the action for the experiment.

        Returns:
            str: The action for the experiment.

        """
        return "disk-write-payload"

    def spec(self, namespace, name) -> dict:
        """
        Returns a dictionary that represents the experiment specification.

        Args:
            namespace (str): The namespace of the experiment.
            name (str): The name of the experiment.

        Returns:
            dict: A dictionary that represents the experiment specification.

        """
        return {
            "action": self.kwargs['action'],
            "address": self.kwargs['address'],
            "disk-write-payload": {
                "size": self.kwargs['size'],
                "path": self.kwargs['path'],
                "payload-process-num": self.kwargs['payload_process_num'],
            },
            "duration": self.kwargs['duration']
        }



from chaosmesh.experiments.base.k8s.podfault.container_kill import BaseContainerKillExperiment


class ContainerKill(BaseContainerKillExperiment):
    """
    The `ContainerKill` class creates a chaosmesh experiment that simulates killing a container.

    This class inherits from the `BaseContainerKillExperiment` and adds the chaosmesh experiment
    functionalities.

    Attributes:
        kwargs (dict): A dictionary of arguments for the experiment.

    """

    def __init__(self, **kwargs):
        """
        Initializes the `ContainerKill` class with the given arguments.

        Args:
            kwargs (dict): A dictionary of arguments for the experiment.

        """
        super(ContainerKill, self).__init__(**kwargs)



from chaosmesh.experiments.base.k8s.stress.cpu import BasePodStressCPUExperiment


class PodStressCPU(BasePodStressCPUExperiment):
    """
    A class that creates a PodStressCPU experiment in ChaosMesh.

    This class is derived from `BasePodStressCPUExperiment` and provides a way to create a PodStressCPU experiment in ChaosMesh.

    Attributes:
        kwargs (dict): A dictionary of arguments for the experiment.

    """

    def __init__(self, **kwargs):
        """
        Initializes the `PodStressCPU` class with the given arguments.

        Args:
            kwargs (dict): A dictionary of arguments for the experiment.

        """
        super(PodStressCPU, self).__init__(**kwargs)



from chaosmesh.experiments.base.k8s.jvmfault.raise_exception import BaseRaiseExceptionExperiment


class RaiseException(BaseRaiseExceptionExperiment):
    """
    Class that represents the RaiseException experiment in ChaosMesh.

    This class is a subclass of `BaseRaiseExceptionExperiment` and inherits all the attributes and methods of its parent class. It can be used to raise an exception in the target container to simulate a failure scenario.

    Attributes:
        kwargs (dict): A dictionary that stores all the experiment's arguments.
    """

    def __init__(self, **kwargs):
        """
        Initializes the `RaiseException` class with the given arguments.

        Args:
            kwargs (dict): A dictionary of arguments for the experiment.

        """
        super(RaiseException, self).__init__(**kwargs)



from abc import ABC
from dataclasses import asdict

from chaosmesh.k8s.experiment import ChaosExperiment
from chaosmesh.k8s.selector import Selector


class PodChaos(ChaosExperiment, ABC):
    """
    Abstract class representing a Pod Chaos experiment.

    This class is used as the base for all the concrete Pod Chaos experiments.
    """

    def __init__(self, **kwargs):
        """
        Initializes the PodChaos instance.

        :param kwargs: Additional arguments to pass to the superclass constructor.
        """
        super(PodChaos, self).__init__(**kwargs)

    @property
    def defaults(self) -> dict:
        """
        Returns the default values for the Pod Chaos experiment.

        :return: The default values for the Pod Chaos experiment.
        :rtype: dict
        """
        return {
            "action": self.action(),
            "gracePeriod": 0,
            "mode": "all",
            "labels": {},
            "pods": {},
            "value": ""
        }

    @property
    def schedule(self) -> dict:
        return {
            "type": "PodChaos",
            "spec": "podChaos"
        }

    def action(self) -> str:
        """
        Returns the action to perform on the selected pods.

        :return: The action to perform on the selected pods.
        :rtype: str
        """
        pass

    def api_resources(self) -> dict:
        """
        Returns the API resources required to perform the Pod Chaos experiment.

        :return: The API resources required to perform the Pod Chaos experiment.
        :rtype: dict
        """
        return {"group": 'chaos-mesh.org', "version": 'v1alpha1', "plural": "podchaos"}

    def validate(self):
        """
        Validates the arguments for the Pod Chaos experiment.

        This method will raise an AssertionError if the required arguments are not present.
        """
        assert self.kwargs['selector'] is not None, "label selector cannot be None"
        assert isinstance(self.kwargs['selector'], Selector), "check the selector type"

    def spec(self, namespace, name) -> dict:
        """
        Returns the specification for the Pod Chaos experiment.

        :param namespace: The namespace where the experiment will be performed.
        :type namespace: str
        :param name: The name of the experiment.
        :type name: str
        :return: The specification for the Pod Chaos experiment.
        :rtype: dict
        """
        return {
            "selector": asdict(self.kwargs['selector']),
            "mode": self.kwargs.get('mode'),
            "value": self.kwargs.get('value'),
            "action": self.kwargs.get('action'),
            "gracePeriod": self.kwargs.get('gracePeriod')
        }



from chaosmesh.experiments.base.k8s.stress.memory import BasePodStressMemoryExperiment


class PodStressMemory(BasePodStressMemoryExperiment):
    """
    The `PodStressMemory` class creates a memory stress experiment in ChaosMesh.

    This class is derived from `BasePodStressMemoryExperiment` and provides all the necessary functionalities for creating and submitting a memory stress experiment to ChaosMesh.

    Args:
        kwargs (dict): A dictionary of arguments for the experiment.

    """

    def __init__(self, **kwargs):
        """
        Initializes the `PodStressMemory` class with the given arguments.

        Args:
            kwargs (dict): A dictionary of arguments for the experiment.

        """
        super(PodStressMemory, self).__init__(**kwargs)



from chaosmesh.experiments.base.hosts.disk.write_payload import BaseWritePayloadExperiment


class WritePayload(BaseWritePayloadExperiment):
    """
    A class for implementing an experiment that writes a payload to a disk.

    Args:
        BaseWritePayloadExperiment (class): A class that represents a base experiment for writing a payload to a disk.

    """

    def __init__(self, **kwargs):
        """
        Initializes a new instance of the WritePayload class.

        Args:
            **kwargs (dict): A dictionary of keyword arguments.

        """
        super(WritePayload, self).__init__(**kwargs)



from abc import ABC

from chaosmesh.experiments.base.hosts.disk import DiskFault


class BaseFillExperiment(DiskFault, ABC):
    """
    A base class for implementing an experiment that fills the disk with data.

    Args:
        DiskFault (class): A class that represents a fault in the disk.
        ABC (class): A class that represents an abstract base class.

    """

    def __init__(self, **kwargs):
        """
        Initializes a new instance of the BaseFillExperiment class.

        Args:
            **kwargs (dict): A dictionary of keyword arguments.

        """
        super(BaseFillExperiment, self).__init__(**kwargs)

    def action(self) -> str:
        """
        Returns the action of the experiment.

        Returns:
            str: The action of the experiment.

        """
        return "disk-fill"

    def validate(self) -> None:
        """
        Validates the keyword arguments passed to the experiment.

        Raises:
            AssertionError: If any of the validation conditions are not met.

        """
        assert self.kwargs['address'] is not None, "address cannot be None"
        assert isinstance(self.kwargs['address'], list), "address should be of type list"

        assert self.kwargs[
                   'size'] is not None, "size cannot be None, the supported formats of the size are: c(=1), w(=2), kB(=1000), K(=1024), MB(=1000*1000), M(=1024*1024), GB and so on"
        assert self.kwargs['path'] is not None, "path cannot be None"

        assert isinstance(self.kwargs['fill_by_fallocate'], bool), "fill_by_fallocate should be of type bool"
        assert self.kwargs['fill_by_fallocate'] is not None, "fill_by_fallocate cannot be None"

    def spec(self, namespace, name) -> dict:
        """
        Returns the specification of the experiment.

        Args:
            namespace (str): The namespace of the experiment.
            name (str): The name of the experiment.

        Returns:
            dict: The specification of the experiment.

        """
        return {
            "action": self.kwargs['action'],
            "address": self.kwargs['address'],
            "disk-fill": {
                "size": self.kwargs['size'],
                "path": self.kwargs['path'],
                "fill-by-fallocate": self.kwargs['fill_by_fallocate'],
            },
            "duration": self.kwargs['duration']
        }



from chaosmesh.experiments import Experiment, v1alpha1
from chaosmesh.experiments.v1alpha1.hosts.disk.fill import Fill
from chaosmesh.experiments.v1alpha1.hosts.disk.read_payload import ReadPayload
from chaosmesh.experiments.v1alpha1.hosts.disk.write_payload import WritePayload
from chaosmesh.experiments.v1alpha1.hosts.stress.cpu import HostsStressCPU
from chaosmesh.experiments.v1alpha1.hosts.stress.memory import HostsStressMemory
from chaosmesh.experiments.v1alpha1.k8s.jvmfault.gc import GC
from chaosmesh.experiments.v1alpha1.k8s.jvmfault.raise_exception import RaiseException
from chaosmesh.experiments.v1alpha1.k8s.network.bandwidth import NetworkBandwidth
from chaosmesh.experiments.v1alpha1.k8s.network.partition import NetworkPartition
from chaosmesh.experiments.v1alpha1.k8s.podfault.container_kill import ContainerKill
from chaosmesh.experiments.v1alpha1.k8s.podfault.pod_failure import PodFailure
from chaosmesh.experiments.v1alpha1.k8s.podfault.pod_kill import PodKill
from chaosmesh.experiments.v1alpha1.k8s.stress.cpu import PodStressCPU
from chaosmesh.experiments.v1alpha1.k8s.stress.memory import PodStressMemory
from chaosmesh.k8s.experiment import ChaosExperiment


class ExperimentFactory:
    """
    ExperimentFactory is a factory class to retrieve ChaosMesh experiments. 
    It provides a unified interface to get different ChaosMesh experiments based on the version.

    :param version: The version of the ChaosMesh supported by the factory.
    :type version: str
    """
    instance = None

    versions = {
        "v1alpha1": {

            # -- kubernetes experiments --

            # pod fault experiments
            Experiment.POD_FAILURE: v1alpha1.k8s.podfault.pod_failure.PodFailure,
            Experiment.POD_KILL: v1alpha1.k8s.podfault.pod_kill.PodKill,
            Experiment.CONTAINER_KILL: v1alpha1.k8s.podfault.container_kill.ContainerKill,

            # k8s stress experiments
            Experiment.POD_STRESS_CPU: v1alpha1.k8s.stress.cpu.PodStressCPU,
            Experiment.POD_STRESS_MEMORY: v1alpha1.k8s.stress.memory.PodStressMemory,

            # k8s jvm fault experiments
            Experiment.RAISE_EXCEPTION: v1alpha1.k8s.jvmfault.raise_exception.RaiseException,
            Experiment.GC: v1alpha1.k8s.jvmfault.gc.GC,

            Experiment.NETWORK_PARTITION: v1alpha1.k8s.network.partition.NetworkPartition,
            Experiment.NETWORK_BANDWIDTH: v1alpha1.k8s.network.bandwidth.NetworkBandwidth,

            # -- kubernetes experiments ends --

            # -- hosts experiments

            # stress
            Experiment.HOST_STRESS_MEMORY: v1alpha1.hosts.stress.memory.HostsStressMemory,
            Experiment.HOST_STRESS_CPU: v1alpha1.hosts.stress.cpu.HostsStressCPU,

            Experiment.HOST_READ_PAYLOAD: v1alpha1.hosts.disk.read_payload.ReadPayload,
            Experiment.HOST_WRITE_PAYLOAD: v1alpha1.hosts.disk.write_payload.WritePayload,
            Experiment.HOST_DISK_FILL: v1alpha1.hosts.disk.fill.Fill
        }
    }

    def __init__(self, version):
        """
        Initialize the ExperimentFactory instance with the given version.

        :param version: The version of the ChaosMesh supported by the factory.
        :type version: str
        """
        assert version is not None, "Version can not be None, ChaosMesh supported versions {list(self.versions.keys())}"

        assert version in self.versions, \
            f"This client does not support ChaosMesh {version}, supported versions {list(self.versions.keys())}"
        self.version = version

    @classmethod
    def get_instance(cls, version):
        """
        Get the singleton instance of the ExperimentFactory.

        :param version: The version of the ChaosMesh supported by the factory.
        :type version: str
        :return: The ExperimentFactory instance.
        :rtype: ExperimentFactory
        """
        if cls.instance is None:
            cls.instance = ExperimentFactory(version)
        return cls.instance

    def get_experiment(self, e: Experiment, **kwargs) -> ChaosExperiment:
        """
        Retrieve a ChaosMesh experiment.

        :param e: The experiment to retrieve.
        :type e: Experiment
        :param kwargs: Additional arguments to pass to the experiment retrieval function.
        :return: The requested ChaosMesh experiment.
        :rtype: object
        """
        return self.versions[self.version][e](**kwargs)



from abc import ABC

from chaosmesh.k8s.experiment import ChaosExperiment


class DiskFault(ChaosExperiment, ABC):
    """
    This class represents a disk fault experiment using the Chaos Experiment framework.
    """

    def __init__(self, **kwargs):
        """
        Initializes a new instance of the DiskFault class.
        :param kwargs: Optional keyword arguments for configuring the experiment.
        """
        super(DiskFault, self).__init__(**kwargs)

    def api_resources(self) -> dict:
        """
        Returns a dictionary containing the Kubernetes API group, version, and plural for the experiment.
        :return: A dictionary containing the API resources for the experiment.
        """
        return {"group": 'chaos-mesh.org', "version": 'v1alpha1', "plural": "physicalmachinechaos"}

    @property
    def schedule(self) -> dict:
        return {
            "type": "DiskFault",
            "spec": "diskFault"
        }

    @property
    def defaults(self) -> dict:
        """
        Returns a dictionary containing default values for the experiment configuration.
        :return: A dictionary containing default values for the experiment configuration.
        """
        return {
            "action": self.action(),
            "duration": "",
        }

    def validate(self) -> None:
        """
        Validates the experiment configuration to ensure that all required parameters are present.
        :raises AssertionError: If any required parameters are missing.
        """
        assert self.kwargs['address'] is not None, "address cannot be None"
        assert isinstance(self.kwargs['address'], list), "address should be of type list"

        assert self.kwargs[
                   'size'] is not None, "size cannot be None, the supported formats of the size are: c(=1), w(=2), kB(=1000), K(=1024), MB(=1000*1000), M(=1024*1024), GB and so on"
        assert self.kwargs['path'] is not None, "path cannot be None"

        assert self.kwargs['payload_process_num'] is not None, "payload_process_num cannot be None"
        assert self.kwargs.get('payload_process_num', 0) > 0, "payload_process_num should be greater than or equal to 1"

    def action(self) -> str:
        """
        Performs the main action of the experiment.
        :return: A string representing the result of the experiment.
        """
        pass



from kubernetes import config


def load_kube_config():
    try:
        config.load_incluster_config()
    except config.ConfigException:
        try:
            config.load_kube_config()
        except config.ConfigException:
            raise Exception("Could not configure kubernetes python client")



from abc import ABC
from dataclasses import asdict

from chaosmesh.experiments.base.k8s.stress import StressTest
from chaosmesh.k8s.selector import Selector


class BasePodStressCPUExperiment(StressTest, ABC):
    """
    BasePodStressCPUExperiment is a class that implements the abstract base class StressTest and is used to perform a stress test on CPU resources of Pods.

    Args:
        **kwargs: The keyword arguments that are passed to the class constructor.

    Attributes:
        kwargs (dict): A dictionary of arguments passed to the class constructor.
    """

    def __init__(self, **kwargs):
        """
        The constructor for the BasePodStressCPUExperiment class. It initializes the attributes of the class by calling the superclass constructor.

        Args:
            **kwargs: The keyword arguments that are passed to the class constructor.
        """
        super(BasePodStressCPUExperiment, self).__init__(**kwargs)

    def validate(self) -> None:
        """
        The validate method is used to validate the arguments passed to the class. It checks if the selector argument is not None and is of type `Selector`. It also checks if the workers and load arguments are not None.

        Raises:
            AssertionError: If the selector argument is None or is not of type `Selector`.
            AssertionError: If the workers argument is None.
            AssertionError: If the load argument is None.
        """
        assert self.kwargs['selector'] is not None, "label selector cannot be None"
        assert isinstance(self.kwargs['selector'], Selector), "check the selector type"

        assert self.kwargs['workers'] is not None, "workers cannot be None"
        assert self.kwargs['load'] is not None, "size cannot be None"

    def spec(self, namespace, name) -> dict:
        """
        The spec method is used to create the specification for the experiment.

        Args:
            namespace (str): The namespace in which the experiment is to be run.
            name (str): The name of the experiment.

        Returns:
            dict: A dictionary that represents the specification of the experiment.
        """
        return {
            "selector": asdict(self.kwargs['selector']),
            "mode": self.kwargs.get('mode'),
            "stressors": {
                "cpu": {
                    "workers": self.kwargs.get('workers'),
                    "load": self.kwargs.get('load')
                }
            },
            "duration": self.kwargs.get('duration')
        }



from chaosmesh.experiments.base.hosts.stress.cpu import BaseHostsStressCPUExperiment


class HostsStressCPU(BaseHostsStressCPUExperiment):
    """
    The `HostsStressCPU` class creates a CPU stress experiment for Kubernetes hosts in Chaos Mesh.

    This class extends the `BaseHostsStressCPUExperiment` class and inherits its properties and methods.
    """

    def __init__(self, **kwargs):
        """
        Initialize the `HostsStressCPU` class by calling the superclass's `__init__` method with the provided arguments.

        :param kwargs: Keyword arguments passed to the superclass's `__init__` method.
        """
        super(HostsStressCPU, self).__init__(**kwargs)



from dataclasses import dataclass


# the naming conversion should in-lined with the k8s resource definition

@dataclass
class Metadata:
    namespace: str
    name: str
    labels: dict

    def __init__(self, namespace: str, name: str, labels: dict) -> None:
        self.namespace = namespace
        self.name = name
        self.labels = labels


@dataclass
class Manifest:
    kind: str
    apiVersion: str
    metadata: Metadata
    spec: dict

    def __init__(self, kind: str, api_version: str, metadata: Metadata, spec: dict) -> None:
        self.kind = kind
        self.apiVersion = api_version
        self.metadata = metadata
        self.spec = spec



from abc import abstractmethod, ABC
from dataclasses import asdict

import time
from kubernetes.client import ApiException
from polling import TimeoutException

from chaosmesh.k8s.k8s_resource import K8SResource
from chaosmesh.k8s.manifest import Manifest, Metadata


class CustomObjectsApi(K8SResource, ABC):
    """
    Class for custom object api
    """

    def __init__(self):
        super(CustomObjectsApi, self).__init__()

    @abstractmethod
    def api_resources(self):
        pass

    @property
    def group(self):
        return self.api_resources().get('group')

    @property
    def version(self):
        return self.api_resources().get('version')

    @property
    def plural(self):
        return self.api_resources().get('plural')

    @abstractmethod
    def spec(self, namespace, name) -> dict:
        pass

    def manifest(self, namespace, name, labels: dict = None) -> dict:
        return asdict(Manifest(
          kind=self.plural,
          api_version=self.group + "/" + self.version,
          metadata=Metadata(namespace=namespace, name=name, labels=labels),
          spec=self.spec(namespace=namespace, name=name)
        ))

    @property
    def client(self):
        """
        Returns the CustomObjectAPI client
        :return: 
        """
        return self.k8sclient.CustomObjectsApi()

    def apply(self, namespace, object):
        return self.client.create_namespaced_custom_object(group=self.group, version=self.version, namespace=namespace, plural=self.plural, body=object, pretty='true')

    def get_clustered(self, name):
        return self.client.get_cluster_custom_object(group=self.group, version=self.version, plural=self.plural, name=name)

    def get(self, namespace, name):
        return self.client.get_namespaced_custom_object(group=self.group, version=self.version, plural=self.plural, name=name, namespace=namespace)

    def list(self, namespace, **kwargs):
        return self.client.list_namespaced_custom_object(group=self.group, version=self.version, namespace=namespace, plural=self.plural, **kwargs)

    def list_clustered(self):
        return self.client.list_cluster_custom_object(group=self.group, version=self.version, plural=self.plural)

    def add_annotation(self, namespace, name, annotations_map):
        return self.client.patch_namespaced_custom_object_with_http_info(group=self.group, version=self.version, namespace=namespace, plural=self.plural, name=name,
                                                                         body={
                                                                             "metadata": {"annotations": annotations_map}
                                                                         })

    def status(self, namespace, name):
        """
        Returns the status of k8s object
        :param namespace: 
        :param name: 
        :return: 
        """
        return self.client.get_namespaced_custom_object_status(group=self.group, version=self.version, namespace=namespace, plural=self.plural, name=name)['status']

    def delete(self, namespace, name, retries=120, foreground=True):
        """
        Delete object in foreground by default
        :param foreground: 
        :param namespace: 
        :param name: 
        :param retries: 
        :return: 
        """
        deleted = None
        try:
            deleted = self.client.delete_namespaced_custom_object(group=self.group, version=self.version, namespace=namespace, plural=self.plural, name=name)
            if foreground:
                while self.get(namespace, name) is not None:
                    retries -= 1
                    if retries == 0:
                        raise TimeoutException("Timeout out deleting the resource")
                    time.sleep(1)
        except ApiException as e:
            if e.status == 404:
                pass
            else:
                raise

        return deleted

    def exists(self, name, namespace):
        try:
            obj = self.get(name=name, namespace=namespace)
            return obj is not None
        except ApiException as e:
            if e.status == 404:
                return False
            else:
                raise



from abc import ABC
from dataclasses import asdict

from chaosmesh.experiments.base.k8s.stress import StressTest
from chaosmesh.k8s.selector import Selector


class BasePodStressMemoryExperiment(StressTest, ABC):
    """
    BasePodStressMemoryExperiment is a subclass of `StressTest` and `ABC` (Abstract Base Class). It is used to define a stress experiment on memory.

    Attributes:
    None

    Methods:
    __init__(self, **kwargs)
        Initializes the `BasePodStressMemoryExperiment` class. It takes in `kwargs` as an argument and calls `super` to initialize the `StressTest` class.

    validate(self)
        Validates the arguments passed to the `BasePodStressMemoryExperiment` class. It checks if the `selector` argument is not None and is of type `Selector`. It also checks if the `workers` and `size` arguments are not None.

    spec(self, namespace, name)
        Returns a dictionary that represents the specification of the stress experiment on memory. The dictionary contains `selector`, `mode`, `stressors`, and `duration`. `selector` is a dictionary representation of the `selector` argument passed to the class. `stressors` is a dictionary containing the `workers` and `size` arguments. `duration` is the duration argument passed to the class.
    """

    def __init__(self, **kwargs):
        super(BasePodStressMemoryExperiment, self).__init__(**kwargs)

    def validate(self) -> None:
        assert self.kwargs['selector'] is not None, "label selector cannot be None"
        assert isinstance(self.kwargs['selector'], Selector), "check the selector type"

        assert self.kwargs['workers'] is not None, "workers cannot be None"
        assert self.kwargs['size'] is not None, "size cannot be None"

    def spec(self, namespace, name) -> dict:
        return {
            "selector": asdict(self.kwargs['selector']),
            "mode": self.kwargs.get('mode'),
            "stressors": {
                "memory": {
                    "workers": self.kwargs.get('workers'),
                    "size": self.kwargs.get('size')
                }
            },
            "duration": self.kwargs.get('duration')
        }



from chaosmesh.experiments.base.k8s.jvmfault.gc import BaseGCExperiment


class GC(BaseGCExperiment):
    """
    GC is a subclass of the `BaseGCExperiment` class. It is used to perform GC experiments in ChaosMesh.

    This class is initialized with the same keyword arguments as the `BaseGCExperiment` class.
    """

    def __init__(self, **kwargs):
        """
        Initialize the GC class. This method calls the `__init__` method of the parent class, `BaseGCExperiment`,
        with the provided keyword arguments.

        Args:
            **kwargs: Keyword arguments to pass to the parent class's `__init__` method.
        """
        super(GC, self).__init__(**kwargs)



from glob import iglob
from pathlib import Path

import semver
from setuptools import find_packages
from setuptools import setup

version_file = "version.txt"

this_directory = Path(__file__).parent
long_description = (this_directory / "../../README.md").read_text()


# NOTE: major or minor version should be manually updated if the changes are NOT backward compatible
def bump_version():
    with open(version_file) as f:
        version = f.read().strip()

    incremented_version = semver.VersionInfo.parse(version).bump_patch()
    with open(version_file, "w") as file:
        file.write(str(incremented_version))

    return str(incremented_version)


def get_requirements():
    with open('requirements.txt') as f:
        return f.read().splitlines()


setup(name='chaos_mesh',
      version=bump_version(),
      description='A client to create experiments in ChaosMesh',
      long_description=long_description,
      long_description_content_type='text/markdown',
      author='Vishrant Gupta',
      author_email='vishrant.gupta@gmail.com',
      packages=find_packages(),
      url='https://github.com/vishrantgupta/chaos-mesh',
      include_package_data=True,
      install_requires=get_requirements(),
      platform='any',
      data_files=[
          ('version.txt', iglob('version.txt', recursive=True)),
      ],
      zip_safe=False)



"""
This module defines the `NetworkChaos` class, which is a sub-class of `ChaosExperiment` from `chaosmesh.k8s.experiment` package.
"""

from abc import ABC

from chaosmesh.k8s.experiment import ChaosExperiment


class NetworkChaos(ChaosExperiment, ABC):
    """
    A class to define the network chaos experiment that extends the `ChaosExperiment` and `ABC` classes.
    """

    def __init__(self, **kwargs):
        """
        Initialize the `NetworkChaos` instance with keyword arguments.

        Args:
            **kwargs: Keyword arguments to configure the network chaos experiment.
        """
        super(NetworkChaos, self).__init__(**kwargs)

    def action(self) -> str:
        """
        Override the `action` method from the `ChaosExperiment` class to implement the network chaos experiment.

        Returns:
            str: A string representation of the network chaos experiment action.
        """
        pass

    @property
    def schedule(self) -> dict:
        return {
            "type": "NetworkChaos",
            "spec": "networkChaos"
        }

    @property
    def defaults(self) -> dict:
        """
        Define the default values for the network chaos experiment.

        Returns:
            dict: A dictionary of default values for the network chaos experiment.
        """
        yield

    def api_resources(self) -> dict:
        """
        Define the API resources for the network chaos experiment.

        Returns:
            dict: A dictionary of API resources for the network chaos experiment.
        """
        return {"group": 'chaos-mesh.org', "version": 'v1alpha1', "plural": "networkchaos"}

    def validate(self) -> None:
        """
        Validate the configuration of the network chaos experiment.
        """
        pass

    def spec(self, namespace, name) -> dict:
        """
        Define the specification of the network chaos experiment.

        Args:
            namespace (str): The namespace of the network chaos experiment.
            name (str): The name of the network chaos experiment.

        Returns:
            dict: A dictionary representing the specification of the network chaos experiment.
        """
        pass



from dataclasses import dataclass

from typing import List, Dict


@dataclass
class Selector:
    """
    Class that represents the selector for selecting objects in a namespace in k8s cluster.

    Attributes:
    ----------------
    namespaces (List[str]): List of namespaces to select objects from.
    labelSelectors (Dict): Dictionary of label selectors to filter the objects.
    pods (List[str]): List of pod names to select.
    """
    namespaces: List[str] = None
    labelSelectors: Dict = None
    pods: List[str] = None



from abc import ABC
from dataclasses import asdict

from chaosmesh.experiments.base.k8s.network import NetworkChaos
from chaosmesh.k8s.selector import Selector


class BaseNetworkPartitionExperiment(NetworkChaos, ABC):
    """
    A network partition experiment for Kubernetes.
    """

    def __init__(self, **kwargs):
        """
        Initialize the NetworkPartitionExperiment.

        :param kwargs: keyword arguments for the experiment.
        """

        super(BaseNetworkPartitionExperiment, self).__init__(**kwargs)

    @property
    def defaults(self):
        """
        Get the default configuration for the experiment.

        :return: default configuration for the experiment.
        """

        return {
            "action": self.action(),
            "duration": "",
            "mode": "all",
        }

    def validate(self) -> None:
        """
        Validate the experiment parameters.
        """

        assert self.kwargs['selector'] is not None, "label selector cannot be None"
        assert isinstance(self.kwargs['selector'], Selector), "invalid Selector type, should be of type k8s.selector"

        assert self.kwargs['external_targets'] is not None, "external_targets cannot be None"
        assert isinstance(self.kwargs['external_targets'], list), "external_targets should be of type list"

        assert self.kwargs['direction'] is not None, "direction cannot be None"

    def action(self) -> str:
        """
        Get the action of the experiment.

        :return: action of the experiment.
        """

        return "partition"

    def spec(self, namespace, name) -> dict:
        """
        Get the specification of the experiment.

        :param namespace: namespace of the experiment.
        :param name: name of the experiment.

        :return: specification of the experiment.
        """

        return {
            "selector": asdict(self.kwargs['selector']),
            "mode": self.kwargs['mode'],
            "action": self.kwargs['action'],
            "duration": self.kwargs['duration'],
            "direction": self.kwargs['direction'],
            "externalTargets": self.kwargs['external_targets'],
        }



from abc import ABC

from chaosmesh.experiments.base.k8s.jvmfault import JVMFault
from chaosmesh.k8s.selector import Selector


class BaseGCExperiment(JVMFault, ABC):
    """
    Base class for GC experiments in ChaosMesh.

    This class provides a basic implementation of a GC experiment in ChaosMesh and serves as a base for other GC experiments. It inherits from `JVMFault` and implements the abstract base class `ABC`.
    :param kwargs: Additional keyword arguments.
    :type kwargs: dict
    """

    def __init__(self, **kwargs):
        """
        Initializes the `BaseGCExperiment` class.

        :param kwargs: Additional keyword arguments.
        :type kwargs: dict
        """
        super(BaseGCExperiment, self).__init__(**kwargs)

    @property
    def defaults(self):
        """
        Returns the default settings for the GC experiment.

        :return: A dictionary containing the default settings.
        :rtype: dict
        """
        return {
            "action": self.action(),
            "class": "",
            "cpuCount": 0,
            "duration": "",
            "exception": "",
            "latency": 0,
            "memType": "",
            "method": "",
            "mode": "all",
            "ruleData": "",
            "value": "",
        }

    def action(self) -> str:
        """
        Returns the action for the GC experiment.

        :return: The action for the GC experiment.
        :rtype: str
        """
        return "gc"

    def validate(self):
        """
        Validates the GC experiment settings.

        This method checks that the required settings (i.e. `selector` and `port`) are not `None` and of the correct type (`Selector`).

        :raises: AssertionError if the required settings are not set or of the incorrect type.
        """
        assert self.kwargs['selector'] is not None, "label selector cannot be None"
        assert isinstance(self.kwargs['selector'], Selector), "invalid Selector type, should be of type k8s.selector"

        assert self.kwargs['port'] is not None, "port cannot be None"



from chaosmesh.experiments.base.k8s.podfault.pod_kill import BasePodKillExperiment


class PodKill(BasePodKillExperiment):
    """
    The `PodKill` class creates a `PodKill` experiment in ChaosMesh.

    The `PodKill` class is a subclass of `BasePodKillExperiment` and provides functionalities to create a `PodKill` experiment in ChaosMesh. The class is initialized with arguments that are passed as a dictionary. The arguments are then passed to the parent class.

    Args:
        kwargs (dict): A dictionary of arguments passed to the experiment.

    """

    def __init__(self, **kwargs):
        """
        Initializes the `PodKill` class.

        This method is called during the instantiation of the class. It initializes the parent class with the given arguments.

        Args:
            kwargs (dict): A dictionary of arguments passed to the parent class.

        """
        super(PodKill, self).__init__(**kwargs)



import json
import logging
from abc import ABC

from polling import poll

from chaosmesh.k8s.chaos_mesh import ChaosMesh

log = logging.getLogger("chaosmesh")


class ChaosExperiment(ChaosMesh, ABC):
    """
    The base class for all ChaosMesh experiments.

    This class provides common functionalities for all ChaosMesh experiments, including
    defaults, validation, submission, pause, and apply.

    Attributes:
        kwargs (dict): A dictionary that stores all the experiment's arguments.

    """

    def __init__(self, **kwargs):
        """
        Initializes the `ChaosExperiment` class with the given arguments.

        Args:
            kwargs (dict): A dictionary of arguments for the experiment.

        """
        super(ChaosExperiment, self).__init__(**kwargs)

    @property
    def schedule(self) -> dict:
        yield

    def _is_injected(self, namespace, name):
        """
        Check if the experiment has been injected.

        Args:
            namespace (str): The namespace of the experiment.
            name (str): The name of the experiment.

        Returns:
            bool: Whether the experiment has been injected or not.

        """
        log.debug(f"checking if experiment {name} in {namespace} namespace got injected")

        obj = self.get(name=name, namespace=namespace)
        if obj is not None:
            conditions = obj['status']['conditions']

            for condition in conditions:
                log.debug(f"chaosmesh experiment {name} status is {condition['status']} for condition type {condition['type']}")

                if condition['type'] == 'AllInjected':
                    records = obj['status']['experiment']['containerRecords']
                    for record in records:
                        if record['phase'] == 'Injected':
                            log.debug(f"chaosmesh experiment {name} got injected")
                            return True
            return False

    def _wait_experiment_injection(self, namespace, name):
        """
        Wait for the experiment to be injected.

        Args:
            namespace (str): The namespace of the experiment.
            name (str): The name of the experiment.

        """
        poll(lambda: self._is_injected(namespace, name),
             timeout=int(120),
             step=2,
             ignore_exceptions=(Exception,))

    def pause(self, namespace, name):
        """
        Pauses a running ChaosMesh experiment.

        This method adds an annotation to the running experiment in order to pause it.

        Args:
            namespace (str): The namespace where the experiment is running.
            name (str): The name of the experiment.

        """
        self.add_annotation(namespace=namespace, name=name, annotations_map={"experiment.chaos-mesh.org/pause": 'true'})

    def create(self, name, namespace, labels=None):
        """
        Submits a ChaosMesh experiment.

        This method creates a manifest for the experiment and applies it to the target namespace.
        It also waits for the experiment to be injected into the target namespace.

        Args:
            name (str): The name of the experiment.
            namespace (str): The namespace where the experiment should be applied.
            labels (dict, optional): A dictionary of labels to be applied to the experiment. Defaults to `None`.

        Returns:
            The applied experiment object.

        """
        manifest = self.manifest(namespace=namespace, name=name, labels=labels)

        log.debug(f"creating chaosmesh resource {json.dumps(manifest, indent=4)}")

        applied = super().apply(namespace=namespace, object=manifest)
        self._wait_experiment_injection(namespace=namespace, name=name)

        log.info(f"chaosmesh experiment {name} got applied")

        return applied
